#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "RescaleFFT", 
	Category = "Value",
	Help = "Rescales the linear FFT bands to a log scale", 
	Tags = "")]
	#endregion PluginInfo
	public class ValueRescaleFFTNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("FFT Input")]
		ISpread<double> FInput;
		
		[Input("Normalize", DefaultValue = 1, IsSingle = true)]
		ISpread<bool> FNormalize;
		
		[Input("Spread Count", DefaultValue = 16.0, IsSingle = true)]
		ISpread<int> FSpreadCount;

		[Output("Output")]
		ISpread<double> FOutput;
		
		[Output("Band Counts")]
		ISpread<double> FBandCountOut;

		[Import()]
		ILogger FLogger;
		
		private int FCount;
		private int FFFTCount;
		private int[] FBandCounts;
		
		#endregion fields & pins
		
		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			//calc band counts
			if(FSpreadCount[0] != FCount  || FFFTCount!=FInput.SliceCount)
			{
				FFFTCount = FInput.SliceCount;
				FCount = Math.Min(FSpreadCount[0], FFFTCount/2);
				
				FBandCounts = new int[FCount];
				
				//get exponential band counts
				for(int i=0; i<FCount; i++)
				{
					FBandCounts[i] = 1 << i;
				}
				
				var factor = FFFTCount / (double)FBandCounts.Sum();
				
				//rescale band counts
				for(int i=0; i<FCount; i++)
				{
					FBandCounts[i] = 
						(int)Math.Max(Math.Round(FBandCounts[i] * factor), 1);
				}
				
				//make sure it fits FFFTCount
				FBandCounts[FCount-1] += FFFTCount - FBandCounts.Sum();
				
				FOutput.SliceCount = FCount;
				FBandCountOut.SliceCount = FCount;
				
				FLogger.Log(LogType.Debug, "hi tty!");
			}
			
			var start = 0;
			for (int i = 0; i < FCount; i++)
			{
				FOutput[i] = FInput.SumRange(start, FBandCounts[i]);
				
				if(FNormalize[0]) FOutput[i] /= FBandCounts[i];
				
				FBandCountOut[i] = FBandCounts[i];
				start += FBandCounts[i];
			}
		}
	}
	
	//helpers
	public static class ArrayExtensions
	{
		public static int Sum(this int[] a)
		{
			int count = a.Length;
			int ret = 0;
			
			for(int i=0; i<count; i++)
			{
				ret += a[i];
			}
			
			return ret;
		}
		
		public static double SumRange(this ISpread<double> a, int start, int count)
		{
			double ret = 0;
			
			for(int i=start; i<start+count; i++)
			{
				ret += a[i];
			}
			
			return ret;
		}
	}
}
